/* eslint-disable no-undef */
/* eslint-disable no-return-assign */

class UIComponent {
  constructor(iframeUrl, className, handleMessage) {
    this.handleMessage = handleMessage;
    this.iframeElement = null;
    this.iframePort = null;
    this.showing = false;
    this.iframeFrameId = null;
    // TODO(philc): Make the @options object default to {} and remove the null checks.
    this.options = null;
    this.shadowDOM = null;

    DomUtils.documentReady(() => {
      const styleSheet = DomUtils.createElement('style');
      styleSheet.type = 'text/css';
      // Default to everything hidden while the stylesheet loads.
      styleSheet.innerHTML = 'iframe {display: none;}';

      chrome.storage.local.get('vimiumCSSInChromeStorage', items => (styleSheet.innerHTML = items.vimiumCSSInChromeStorage));
      this.iframeElement = DomUtils.createElement('iframe');
      Object.assign(this.iframeElement, {
        className,
        seamless: 'seamless',
      });

      // 创建shadowDOM
      const shadowWrapper = DomUtils.createElement('div');
      // Firefox < 63 不支持createShadow
      // https://hacks.mozilla.org/2018/10/firefox-63-tricks-and-treats/
      if (shadowWrapper.attachShadow) {
        this.shadowDOM = shadowWrapper.attachShadow({ mode: 'open' });
      } else {
        this.shadowDOM = shadowWrapper;
      }
      // append样式以及iframe
      this.shadowDOM.appendChild(styleSheet);
      this.shadowDOM.appendChild(this.iframeElement);
      this.handleDarkReaderFilter();
      // 隐藏iframe
      this.toggleIframeElementClasses('vimiumUIComponentVisible', 'vimiumUIComponentHidden');

      // Open a port and pass it to the iframe via window.postMessage.  We use an AsyncDataFetcher to handle
      // requests which arrive before the iframe (and its message handlers) have completed initialization.  See
      // #1679.
      this.iframePort = new AsyncDataFetcher(setIframePort => {
        // We set the iframe source and append the new element here (as opposed to above) to avoid a potential
        // race condition vis-a-vis the "load" event (because this callback runs on "nextTick").
        this.iframeElement.src = chrome.runtime.getURL(iframeUrl) + '?referer=' + encodeURIComponent(location.href);
        document.documentElement.appendChild(shadowWrapper);

        this.iframeElement.addEventListener('load', () => {
          // Get vimiumSecret so the iframe can determine that our message isn't the page impersonating us.
          chrome.storage.local.get('vimiumSecret', ({ vimiumSecret }) => {
            const { port1, port2 } = new MessageChannel();
            this.iframeElement.contentWindow.postMessage(vimiumSecret, chrome.runtime.getURL(''), [port2]);
            port1.onmessage = event => {
              let eventName = null;
              if (event) {
                eventName = (event.data ? event.data.name : undefined) || event.data;
              }

              switch (eventName) {
                case 'uiComponentIsReady':
                  // If any other frame receives the focus, then hide the UI component.
                  chrome.runtime.onMessage.addListener(({ name, focusFrameId }) => {
                    if (name === 'frameFocused' && this.options && this.options.focus && ![frameId, this.iframeFrameId].includes(focusFrameId)) {
                      this.hide(false);
                    }
                    // We will not be calling sendResponse.
                    return false;
                  });
                  // If this frame receives the focus, then hide the UI component.
                  /*
                    问题描述：如果是help 点击链接，搜索框会消失，无法显示内置页面
                    修改理由：允许用户在搜索框下操作网页
                    修改事件为click,实际测试只有点击搜索框外部页面才会触发click事件
                   */
                  window.addEventListener(
                    'click',
                    forTrusted(event => {
                      if (event.view === window && this.options && this.options.focus) {
                        this.hide(false);
                      }
                      // Continue propagating the event.
                      return true;
                    }),
                    true
                  );
                  // Set the iframe's port, thereby rendering the UI component ready.
                  setIframePort(port1);
                  break;
                case 'setIframeFrameId':
                  this.iframeFrameId = event.data.iframeFrameId;
                  break;
                case 'hide':
                  return this.hide();
                default:
                  this.handleMessage(event);
              }
            };
          });
        });
      });
      if (Utils.isFirefox()) {
        this.postMessage({ name: 'settings', isFirefox: true });
      }
    });
  }

  // 暗黑模式处理，即使使用了DarkReader
  handleDarkReaderFilter() {
    const reverseFilterClass = 'reverseDarkReaderFilter';

    const reverseFilterIfExists = () => {
      // The DarkReader extension creates this element if it's actively modifying the current page.
      const darkReaderElement = document.getElementById('dark-reader-style');
      if (darkReaderElement && darkReaderElement.innerHTML.includes('filter')) {
        this.iframeElement.classList.add(reverseFilterClass);
      } else {
        this.iframeElement.classList.remove(reverseFilterClass);
      }
    };
    reverseFilterIfExists();
    const observer = new MutationObserver(reverseFilterIfExists);
    observer.observe(document.head, { characterData: true, subtree: true, childList: true });
  }

  // 切换iframe的显示
  toggleIframeElementClasses(removeClass, addClass) {
    this.iframeElement.classList.remove(removeClass);
    this.iframeElement.classList.add(addClass);
  }

  // Post a message (if provided), then call continuation (if provided).  We wait for documentReady() to ensure
  // that the @iframePort set (so that we can use @iframePort.use()).
  postMessage(message = null, continuation = null) {
    if (!this.iframePort) {
      return;
    }

    this.iframePort.use(function(port) {
      if (message != null) {
        port.postMessage(message);
      }
      if (continuation) {
        continuation();
      }
    });
  }

  activate(options = null) {
    this.options = options;
    this.postMessage(this.options, () => {
      this.toggleIframeElementClasses('vimiumUIComponentHidden', 'vimiumUIComponentVisible');
      if (this.options && this.options.focus) {
        this.iframeElement.focus();
      }
      this.showing = true;
    });
  }

  hide(shouldRefocusOriginalFrame) {
    // We post a non-message (null) to ensure that hide() requests cannot overtake activate() requests.
    if (shouldRefocusOriginalFrame == null) {
      shouldRefocusOriginalFrame = true;
    }
    this.postMessage(null, () => {
      if (!this.showing) {
        return;
      }
      this.showing = false;
      this.toggleIframeElementClasses('vimiumUIComponentVisible', 'vimiumUIComponentHidden');
      if (this.options && this.options.focus) {
        this.iframeElement.blur();
        if (shouldRefocusOriginalFrame) {
          if (this.options && this.options.sourceFrameId != null) {
            chrome.runtime.sendMessage({
              handler: 'sendMessageToFrames',
              message: { name: 'focusFrame', frameId: this.options.sourceFrameId, forceFocusThisFrame: true },
            });
          } else {
            Utils.nextTick(() => window.focus());
          }
        }
      }
      this.options = null;
      this.postMessage('hidden'); // Inform the UI component that it is hidden.
    });
  }
}

global.UIComponent = UIComponent;
